/*  Simple template class for quaternions
 *  In this file, there are some little mathtests
 *  Copyright (C) R. Bourquin, 2008
 *  License: GNU GPL v2.0 or above
 */

#include <iostream>
#include <cstring>
#include "quaternion.hh"

template <class T>
void print_quater(Quaternion<T> q, std::string s) {
    std::cout << "Quaternion " << s << " :\t\t" << q.get_t()<<"+"<<q.get_u()<<"*I+"<<q.get_v()<<"*J+"<<q.get_w()<<"*K" << std::endl;
}

int main() {
    // Quaternion basis elements
    Quaternion<double> i = Quaternion<double>(0.0, 1.0, 0.0, 0.0);
    Quaternion<double> j = Quaternion<double>(0.0, 0.0, 1.0, 0.0);
    Quaternion<double> k = Quaternion<double>(0.0, 0.0, 0.0, 1.0);
    print_quater<double>(i,"I");
    print_quater<double>(j,"J");
    print_quater<double>(k,"K");

    // fundamental formula for quaternion multiplication i^2 = j^2 = k^2 = i*j*k = −1
    Quaternion<double> is = i*i;
    Quaternion<double> js = j*j;
    Quaternion<double> ks = k*k;
    Quaternion<double> ijk = i*j*k;
    print_quater<double>(is,"I^2");
    print_quater<double>(js,"J^2");
    print_quater<double>(ks,"K^2");
    print_quater<double>(ijk,"I*J*K");

    Quaternion<double> q1 = Quaternion<double>(1.0, 2.0, 3.0, 4.0);
    Quaternion<double> q2 = Quaternion<double>(5.0, 6.0, 7.0, 8.0);

    print_quater<double>(q1,"Q1");
    print_quater<double>(q2,"Q2");

    Quaternion<double> q3 = q1+q2;
    print_quater<double>(q3,"Q1+Q2");

    Quaternion<double> q4 = q1-q2;
    print_quater<double>(q4,"Q1-Q2");

    Quaternion<double> q5 = q1*q2;
    print_quater<double>(q5,"Q1*Q2");

    Quaternion<double> q6 = q2*q1;
    print_quater<double>(q6,"Q2*Q1");

    Quaternion<double> q7 = q1*3.0;
    print_quater<double>(q7,"Q1*3");

    Quaternion<double> q8 = 3.0*q1;
    print_quater<double>(q8,"3*Q1");

    Quaternion<double> q9 = conjugate(q1);
    print_quater<double>(q9,"conj(Q1)");

    Quaternion<double> q10 = scalar(q2);
    Quaternion<double> q11 = vector(q2);
    print_quater<double>(q10,"scalar");
    print_quater<double>(q11,"vector");

    double l = abs(q1);
    std::cout << "abs: " << l << std::endl;


    Quaternion<double> q12 = inverse(q1);
    print_quater<double>(q12,"Q1^-1");
    Quaternion<double> q13 = q12*q1;
    print_quater<double>(q13,"Q1^-1*Q1");
    Quaternion<double> q14 = q1*q12;
    print_quater<double>(q14,"Q1*Q1^-1");


    return 0;
}
